from registration.backends.default import DefaultBackend as StdDefaultBackend

from registration_names.forms import RegistrationNameControlForm


class DefaultBackend(StdDefaultBackend):
    """
    A registration backend which inherited from django-registration
    `DefaultBackend` and replaces the registration form with
    `RegistrationNameControlForm`.
    """

    def get_form_class(self, request):
        return RegistrationNameControlForm


"""
URLconf which uses django-registration's one but changes backend argument
to `registration_names.backends.simple.SimpleBackend`.
"""


from __future__ import absolute_import
from registration.backends.default.urls import urlpatterns as std_urlpatterns

from ..utils import transform_registration_patters


urlpatterns = transform_registration_patters(std_urlpatterns,
    'registration_names.backends.simple.SimpleBackend')



import os
import sys
sys.path.append(os.getcwd())
os.environ['DJANGO_SETTINGS_MODULE'] = 'registration_names.settings'

from unittest import TestCase

from django.core.exceptions import ImproperlyConfigured
#from django.conf import settings

from checkers import Checker


class IncorrectConfigTests(TestCase):
    def setUp(self):
        pass

    def test_empty_dic(self):
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({})
        self.assertEqual(
            e.exception.message,
            "'control_type' key is expected. "
            "Possible values: 'allowed', 'prohibited', "
            "'allowed_and_prohibited' and 'disabled'.")

    def test_control_type_empty(self):
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': ''})
        self.assertEqual(
            e.exception.message,
            "'control_type' possible values: 'allowed', 'prohibited', "
            "'allowed_and_prohibited' and 'disabled'.")

        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': None})
        self.assertEqual(
            e.exception.message,
            "'control_type' possible values: 'allowed', 'prohibited', "
            "'allowed_and_prohibited' and 'disabled'.")

    def test_control_type_invalid_type(self):
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': 123})
        self.assertEqual(
            e.exception.message,
            "'control_type' possible values: 'allowed', 'prohibited', "
            "'allowed_and_prohibited' and 'disabled'.")

    def test_allowed_wo_list(self):
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': 'allowed'})
        self.assertEqual(
            e.exception.message,
            "'control_type' set to 'allowed' but 'allowed' list not found.")

        # With prohibited-list
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': 'allowed', 'prohibited': []})
        self.assertEqual(
            e.exception.message,
            "'control_type' set to 'allowed' but 'allowed' list not found.")

        # With prohibited-list
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({
                'control_type': 'allowed_and_prohibited',
                'prohibited': []
            })
        self.assertEqual(
            e.exception.message,
            "'control_type' set to 'allowed_and_prohibited' "
            "but 'allowed' list not found.")

    def test_allowed_incorrect_list_type(self):
        # Incorrect type instead of list.
        root = {
            'control_type': 'allowed',
            'allowed': 123,
        }
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker(root)
        self.assertEqual(
            e.exception.message,
            "The value of 'allowed' must be an iterable sequence "
            "(list, tuple). '123' given.")

        # String instead of list.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': 'allowed', 'allowed': 'Name'})
        self.assertEqual(
            e.exception.message,
            "The value of 'allowed' must be an sequence (list, tuple), "
            "not a single string.")

    def test_prohibited_wo_list(self):
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': 'prohibited'})
        self.assertEqual(
            e.exception.message,
            "'control_type' set to 'prohibited' "
            "but 'prohibited' list not found.")

        # With allowed-list.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': 'prohibited', 'allowed': []})
        self.assertEqual(
            e.exception.message,
            "'control_type' set to 'prohibited' "
            "but 'prohibited' list not found.")

        # With allowed-list.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': 'allowed_and_prohibited', 'allowed': []})
        self.assertEqual(
            e.exception.message,
            "'control_type' set to 'allowed_and_prohibited' "
            "but 'prohibited' list not found.")

    def test_prohibited_incorrect_list_type(self):
        # Incorrect type instead of list.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({
                'control_type': 'prohibited',
                'prohibited': 123,
            })
        self.assertEqual(
            e.exception.message,
            "The value of 'prohibited' must be an iterable sequence "
            "(list, tuple). '123' given.")

        # String instead of list.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': 'prohibited', 'prohibited': 'Name'})
        self.assertEqual(
            e.exception.message,
            "The value of 'prohibited' must be an sequence (list, tuple), "
            "not a single string.")

    def test_allowed_and_prohibited(self):
        # Without any list.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': 'allowed_and_prohibited'})
        self.assertEqual(
            e.exception.message,
            "'control_type' set to 'allowed_and_prohibited' "
            "but 'allowed' list not found.")

        # Without prohibited list.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({
                'control_type': 'allowed_and_prohibited',
                'allowed': [],
            })
        self.assertEqual(
            e.exception.message,
            "'control_type' set to 'allowed_and_prohibited' "
            "but 'prohibited' list not found.")

        # Without allowed list.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({
                'control_type': 'allowed_and_prohibited',
                'prohibited': [],
            })
        self.assertEqual(
            e.exception.message,
            "'control_type' set to 'allowed_and_prohibited' "
            "but 'allowed' list not found.")

    def test_str_and_re(self):
        # Non-string ang non-tuple/list.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': 'prohibited', 'prohibited': [123]})
        self.assertEqual(
            e.exception.message,
            "Elements of 'prohibited' must be strings or 3-element tuples "
            "or lists. Element on position 0: '123'.")

        # Incorrect tuple length.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': 'prohibited', 'prohibited': [(1,)]})
        self.assertEqual(
            e.exception.message,
            "3-element tuple is expected in 'prohibited' on position 0, "
            "'(1,)' given.")

        # Incorrect 1st element.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({'control_type': 'prohibited', 'prohibited': [[1, 2, 3]]})
        self.assertEqual(
            e.exception.message,
            "First element of tuple in 'prohibited' on position 0 "
            "must be 're', '1' given.")

        # Incorrect 2nd element.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({
                'control_type': 'prohibited',
                'prohibited': [['re', 2, 3]]
            })
        self.assertEqual(
            e.exception.message,
            "Second element of tuple in 'prohibited' on position 0 must be "
            "a string with one of the keys: 'i'. '2' given.")

        # Incorrect 3rd element.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({
                'control_type': 'prohibited',
                'prohibited': [['re', '', 3]]
            })
        self.assertEqual(
            e.exception.message,
            "Third element of tuple in 'prohibited' on "
            "position 0 must be a string.")

        # Unknown keys.
        with self.assertRaises(ImproperlyConfigured) as e:
            Checker({
                'control_type': 'prohibited',
                'prohibited': [['re', 'qwe', 'pattern']]
            })
        self.assertEqual(
            e.exception.message,
            "Unknown key 'q' in 'prohibited' on position 0. "
            "Possible keys: 'i'.")


class CorrectConfigTests(TestCase):
    def test_disabled(self):
        c = Checker({'control_type': 'disabled'})
        self.assertEqual(c.check("Some name."), True)

        # Strange keys.
        c = Checker({'control_type': 'disabled', 'asdasd': None})
        self.assertEqual(c.check("Some name."), True)

        # Error in lists.
        c = Checker({'control_type': 'disabled', 'allowed': None})
        self.assertEqual(c.check("Some name."), True)

        # Prohibited list.
        root = {
            'control_type': 'disabled',
            'allowed': [],
            'prohibited': ["Some name."]
        }
        c = Checker(root)
        self.assertEqual(c.check("Some name."), True)

    def test_allowed_only(self):
        root = {
            'control_type': 'allowed',
            'allowed': [
                'Explicit name.',
                ('re', '', 'pA+ttern'),
                ('re', 'i', 'sTrAnGe'),
            ]
        }
        c = Checker(root)
        self.assertEqual(c.check("not allowed"), False)
        self.assertEqual(c.check("pattern"), False)  # Lower-case 'a'.

        self.assertEqual(c.check("Explicit name."), True)
        self.assertEqual(c.check("pAttern"), True)
        self.assertEqual(c.check("pAAAttern"), True)

        self.assertEqual(c.check("strange"), True)
        self.assertEqual(c.check("STRANGE"), True)

    def test_prohibited_only(self):
        root = {
            'control_type': 'prohibited',
            'prohibited': [
                'Explicit name.',
                ('re', '', 'pA+ttern'),
                ('re', 'i', 'sTrAnGe'),
            ]
        }
        c = Checker(root)
        self.assertEqual(c.check("allowed"), True)
        self.assertEqual(c.check("pattern"), True)  # Lower-case 'a'.

        self.assertEqual(c.check("Explicit name."), False)
        self.assertEqual(c.check("pAttern"), False)
        self.assertEqual(c.check("pAAAttern"), False)

        self.assertEqual(c.check("strange"), False)
        self.assertEqual(c.check("STRANGE"), False)

    def test_allowed_and_prohibited(self):
        root = {
            'control_type': 'allowed_and_prohibited',
            'allowed': [
                'Explicit name.',
                ('re', '', 'pA+ttern'),
                ('re', 'i', 'sTrAnGe'),
            ],
            'prohibited': [
                'pAAttern',
                ('re', '', 'sT.*')
            ]
        }
        c = Checker(root)
        self.assertEqual(c.check("not allowed"), False)
        self.assertEqual(c.check("pattern"), False)  # Lower-case 'a'.

        self.assertEqual(c.check("Explicit name."), True)
        self.assertEqual(c.check("pAttern"), True)
        self.assertEqual(c.check("pAAttern"), False)
        self.assertEqual(c.check("pAAAttern"), True)

        self.assertEqual(c.check("sTrange"), False)
        self.assertEqual(c.check("strange"), True)
        self.assertEqual(c.check("STRANGE"), True)



from copy import deepcopy
from django.core.urlresolvers import RegexURLPattern
from django.conf.urls import url


def transform_registration_patters(patterns, new_backend):
    """
    Transform patterns declared in registration app to work with
    specified backend.
    """
    result = []
    for p in patterns:
        # Don't transform RegexURLResolver and ones which don't have 'backend'
        # argument specified.
        if (not isinstance(p, RegexURLPattern) or
            'backend' not in p.default_args):
            result.append(p)
            continue

        new_default_args = deepcopy(p.default_args)
        new_default_args['backend'] = new_backend        
        result.append(
            url(p.regex.pattern, p.callback, new_default_args, p.name))
    return result


from registration.backends.simple import SimpleBackend as StdSimpleBackend

from registration_names.forms import RegistrationNameControlForm


class SimpleBackend(StdSimpleBackend):
    """
    A registration backend which inherited from django-registration
    `SimpleBackend` and replaces the registration form with
    `RegistrationNameControlForm`.
    """
    
    def get_form_class(self, request):
        return RegistrationNameControlForm


"""
Usersnames checkers.
"""
import re

from django.core.exceptions import ImproperlyConfigured
from django.utils import six


class Checker(object):
    """
    The checker of usernames allowed.

    Initialized with a dictionary in the following format:

        root = {
            'control_type': 'allowed',

            'allowed': [
                'SimpleAllowed',
                ('re', 'i', 'Regexp!+allowed'),
            ],

            'prohibited': [
                'SimpleProhibited',
            ]
        }

    The first key is 'control_type' which can be 'disabled', 'allowed',
    'prohibited' or 'allowed_and_prohibited'.
    * With 'disabled' no check will be done, all passed usernames will be
    considered allowed.
    * With 'allowed' the 'allowed' list will be used to check if passed
    username is allowed.
    * With 'prohibited' all names will be considered allowed except those which
    explicitly prohibited by 'prohibited' list.
    * With 'allowed_and_prohibited' an username will be considered allowed only
    if it's aexplicitly llowed by 'allowed' list and not explicitly prohibited
    by 'prohibited' list.

    With each 'control_type' all correspondeing lists are required even in case
    they're empty.

    The format of the lists is the following. Each lists consists of two types
    of items: strings and regular expressions. Usersnames will be simply
    compared with string and matched with regexps. Regexps are set by 3-element
    tuples of strings:
    * The first element is a marker 're'.
    * The second element is keys. The only possible key now is 'i' of case
    insensitivity.
    * The third is a regexp pattern.

    Of course, it's possible to use 3-element lists or another suitable type
    instead of tuple.
    """

    __CONTROL_TYPE = 'control_type'
    __CONTROL_TYPE_ALLOWED = 'allowed'
    __CONTROL_TYPE_PROHIBITED = 'prohibited'
    __CONTROL_TYPE_ALLOWED_PROHIBITED = 'allowed_and_prohibited'
    __CONTROL_TYPE_DISABLED = 'disabled'

    def __init__(self, root=None):
        """
        Constructor.

        `root` - a configuration dictionary (e.g. from settings).
        The format is discribed early.
        """
        self.__control_type = None

        if root is None:
            return

        KEY_ALLOWED = 'allowed'
        KEY_PROHIBITED = 'prohibited'

        POSSIBLE_CONTROL_TYPES_STR = "'{}', '{}', '{}' and '{}'".format(
            self.__CONTROL_TYPE_ALLOWED,
            self.__CONTROL_TYPE_PROHIBITED,
            self.__CONTROL_TYPE_ALLOWED_PROHIBITED,
            self.__CONTROL_TYPE_DISABLED)

        if self.__CONTROL_TYPE not in root:
            raise ImproperlyConfigured(
                "'{}' key is expected. "
                "Possible values: {}.".format(
                    self.__CONTROL_TYPE, POSSIBLE_CONTROL_TYPES_STR))

        self.__control_type = root[self.__CONTROL_TYPE]

        if self.__control_type not in [self.__CONTROL_TYPE_ALLOWED,
                                       self.__CONTROL_TYPE_PROHIBITED,
                                       self.__CONTROL_TYPE_ALLOWED_PROHIBITED,
                                       self.__CONTROL_TYPE_DISABLED]:
            raise ImproperlyConfigured(
                "'{}' possible values: "
                "{}.".format(self.__CONTROL_TYPE, POSSIBLE_CONTROL_TYPES_STR))

        self.__allowed_str = None
        self.__allowed_re = None
        self.__prohibited_str = None
        self.__prohibited_re = None

        if (self.__control_type == self.__CONTROL_TYPE_ALLOWED or
                self.__control_type == self.__CONTROL_TYPE_ALLOWED_PROHIBITED):
            if KEY_ALLOWED not in root:
                raise ImproperlyConfigured(
                    "'{}' set to '{}' but '{}' "
                    "list not found.".format(self.__CONTROL_TYPE,
                                             root[self.__CONTROL_TYPE],
                                             KEY_ALLOWED))
            self.__allowed_str, self.__allowed_re = self.__parse_list(
                root[KEY_ALLOWED], KEY_ALLOWED)

        if (self.__control_type == self.__CONTROL_TYPE_PROHIBITED or
                self.__control_type == self.__CONTROL_TYPE_ALLOWED_PROHIBITED):
            if KEY_PROHIBITED not in root:
                raise ImproperlyConfigured(
                    "'{}' set to '{}' but '{}' "
                    "list not found.".format(self.__CONTROL_TYPE,
                                             self.__control_type,
                                             KEY_PROHIBITED))
            self.__prohibited_str, self.__prohibited_re = self.__parse_list(
                root[KEY_PROHIBITED], KEY_PROHIBITED)

    def __parse_list_element(self, list_name, element, element_n):
        """
        Parse element from the list with all necessary checks.

        `list_name` - the name of the list ('allowed', 'prohibited' or
        'allowed_and_prohibited').
        `element` - the element from the list to parse.
        `element_n` - the element's position in the list.
        """

        KEYS_STR = "'i'"

        if isinstance(element, six.string_types):
            return ('str', element,)

        # Element isn't a string - consider it as 3-element re-tuple or list.
        try:
            length = len(element)
        except TypeError:
            raise ImproperlyConfigured(
                "Elements of '{}' must be strings or 3-element "
                "tuples or lists. Element on position {}: '{}'.".format(
                    list_name, element_n, element))

        if length != 3:
            raise ImproperlyConfigured(
                "3-element tuple is expected in '{}' on position {}, "
                "'{}' given.".format(list_name, element_n, element))

        # The first must be a 're' string.
        if element[0] != 're':
            raise ImproperlyConfigured(
                "First element of tuple in '{}' on position {} must be 're', "
                "'{}' given.".format(list_name, element_n, element[0]))

        # The second must be a string with allowed keys.
        if not isinstance(element[1], six.string_types):
            raise ImproperlyConfigured(
                "Second element of tuple in '{}' on position {} must be "
                "a string with one of the keys: {}. '{}' given."
                "".format(list_name, element_n, KEYS_STR, element[1]))

        # The third must be a string.
        if not isinstance(element[2], six.string_types):
            raise ImproperlyConfigured(
                "Third element of tuple in '{}' on position {} must be "
                "a string.".format(list_name, element_n, KEYS_STR))

        re_flags = 0
        for k in element[1]:
            if k == 'i':
                re_flags |= re.I
            else:
                raise ImproperlyConfigured(
                    "Unknown key '{}' in '{}' on position {}. "
                    "Possible keys: {}."
                    "".format(k, list_name, element_n, KEYS_STR))

        return ('re', re.compile(element[2], re_flags),)

    def __parse_list(self, patterns_list, list_name):
        """
        Parse 'allowed', 'prohibited' or 'allowed_and_prohibited' list.

        `patterns_list` - the list.
        `list_name` - the name of the list.
        """

        # Common error - a string instead of a sequence of strings.
        if isinstance(patterns_list, six.string_types):
            raise ImproperlyConfigured(
                "The value of '{}' must be an sequence (list, tuple), "
                "not a single string.".format(list_name))

        # Catch non-iterables.
        try:
            iter(patterns_list)
        except TypeError:
            raise ImproperlyConfigured(
                "The value of '{}' must be an iterable sequence "
                "(list, tuple). '{}' given.".format(list_name, patterns_list))

        result_str = []
        result_re = []
        for i, p in enumerate(patterns_list):
            t, v = self.__parse_list_element(list_name, p, i)
            if t == 're':
                result_re.append(v)
            else:
                result_str.append(v)
        return result_str, result_re

    def __check_allowed(self):
        """
        Determine if checking of allowed is required.
        """

        return (self.__control_type == self.__CONTROL_TYPE_ALLOWED or
                self.__control_type == self.__CONTROL_TYPE_ALLOWED_PROHIBITED)

    def __check_prohibited(self):
        """
        Determine if checking of prohibited is required.
        """

        return (self.__control_type == self.__CONTROL_TYPE_PROHIBITED or
                self.__control_type == self.__CONTROL_TYPE_ALLOWED_PROHIBITED)

    def check(self, value):
        """
        Check passed `value` according to the checker's configuration.
        """

        if self.__control_type == self.__CONTROL_TYPE_DISABLED:
            return True

        allowed = True
        if self.__check_allowed():
            allowed = (value in self.__allowed_str)
            if not allowed:
                for r in self.__allowed_re:
                    if r.match(value):
                        allowed = True
                        break
        if not allowed:
            return False

        prohibited = False
        if self.__check_prohibited():
            prohibited = (value in self.__prohibited_str)
            if not prohibited:
                for r in self.__prohibited_re:
                    if r.match(value):
                        prohibited = True
                        break
        if prohibited:
            return False

        return True



from django.conf import settings
from django import forms
from django.utils.translation import ugettext_lazy as _

from registration.forms import RegistrationForm

from checkers import Checker


class RegistrationNameControlForm(RegistrationForm):
    """
    Form for registering a new user account which checks username to be allowed
    for registeration.
    """

    def clean_username(self):
        ROOT_CONFIG = 'REGISTRATION_NAMES'

        checker = Checker(getattr(settings, ROOT_CONFIG, None))
        username = super(RegistrationNameControlForm, self).clean_username()

        if checker.check(username):
            return username
        else:
            raise forms.ValidationError(_("This username isn't allowed."))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup
from setuptools import find_packages
import os

from registration_names import VERSION


# Allow setup.py to be run from any path.
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

NAME = "django-registration-names"

DESCRIPTION = "A way to control allowed and prohibited user names for registration with django-registration."
LONG_DESCRIPTION = """
A way to control allowed and prohibited user names for registration with
*django-registration*.

The application provides tools (forms, backends, predefined URLs)
to help you define which user names are allowed to register in your project
and which are prohibited.

You may want to prohibit registration of user names which are "system" in
your project, e.g. 'add', 'user', 'profile' etc.
"""

AUTHOR = "Ivan Yurchenko <ivan0yurchenko@gmail.com>"
AUTHOR_EMAIL = "ivan0yurchenko@gmail.com"

MAINTAINER = AUTHOR
MAINTAINER_EMAIL = AUTHOR_EMAIL

URL = 'https://github.com/ivanyu/django-registration-names'

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    keywords="django, django-registration",
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    maintainer=MAINTAINER,
    maintainer_email=MAINTAINER_EMAIL,
    url=URL,
    license="MIT License",
    packages=find_packages(),
    zip_safe=False,
    install_requires=['django', 'django-registration',],
    include_package_data=True,
    classifiers=[
        "Framework :: Django",
        "Programming Language :: Python",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    test_suite="registration_names.tests",
)
